#
# spec file for package ghc-fin
#
# Copyright (c) 2018 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%global pkg_name fin
%bcond_with tests
Name:           ghc-%{pkg_name}
Version:        0.0.1
Release:        4.10
Summary:        Nat and Fin: peano naturals and finite numbers
License:        BSD-3-Clause
Group:          Development/Libraries/Haskell
URL:            https://hackage.haskell.org/package/%{pkg_name}
Source0:        https://hackage.haskell.org/package/%{pkg_name}-%{version}/%{pkg_name}-%{version}.tar.gz
Source1:        https://hackage.haskell.org/package/%{pkg_name}-%{version}/revision/1.cabal#/%{pkg_name}.cabal
BuildRequires:  ghc-Cabal-devel
BuildRequires:  ghc-deepseq-devel
BuildRequires:  ghc-hashable-devel
BuildRequires:  ghc-rpm-macros
%if %{with tests}
BuildRequires:  ghc-inspection-testing-devel
BuildRequires:  ghc-tagged-devel
%endif

%description
This package provides two simple types, and some tools to work with them.
Also on type level as 'DataKinds'.

' -- Peano naturals data Nat = Z | S Nat

-- Finite naturals data Fin (n :: Nat) where Z :: Fin ('S n) S :: Fin n -> Fin
('Nat.S n) '

[vec](https://hackage.haskell.org/package/vec) implements length-indexed
(sized) lists using this package for indexes.

The "Data.Fin.Enum" module let's work generically with enumerations.

See [Hasochism: the pleasure and pain of dependently typed haskell
programming](https://doi.org/10.1145/2503778.2503786) by Sam Lindley and Conor
McBride for answers to /how/ and /why/. Read [APLicative Programming with
Naperian Functors](https://doi.org/10.1007/978-3-662-54434-1_21) by Jeremy
Gibbons for (not so) different ones.

=== Similar packages

* [finite-typelits](https://hackage.haskell.org/package/finite-typelits) .
Is a great package, but uses 'GHC.TypeLits'.

* [type-natural](https://hackage.haskell.org/package/type-natural) depends on
'singletons' package. 'fin' will try to stay light on the dependencies, and
support as many GHC versions as practical.

* [peano](https://hackage.haskell.org/package/peano) is very incomplete

* [nat](https://hackage.haskell.org/package/nat) as well.

* [PeanoWitnesses](https://hackage.haskell.org/package/PeanoWitnesses) doesn't
use 'DataKinds'.

* [type-combinators](https://hackage.haskell.org/package/type-combinators) is
big package too.

%package devel
Summary:        Haskell %{pkg_name} library development files
Group:          Development/Libraries/Haskell
Requires:       %{name} = %{version}-%{release}
Requires:       ghc-compiler = %{ghc_version}
Requires(post): ghc-compiler = %{ghc_version}
Requires(postun): ghc-compiler = %{ghc_version}

%description devel
This package provides the Haskell %{pkg_name} library development files.

%prep
%setup -q -n %{pkg_name}-%{version}
cp -p %{SOURCE1} %{pkg_name}.cabal

%build
%ghc_lib_build

%install
%ghc_lib_install

%check
%cabal_test

%post devel
%ghc_pkg_recache

%postun devel
%ghc_pkg_recache

%files -f %{name}.files
%license LICENSE

%files devel -f %{name}-devel.files
%doc ChangeLog.md

%changelog
* Tue Jul 10 2018 psimons@suse.com
- Add fin at version 0.0.1.
